;; Generated package description from iso-date.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "iso-date" "1.2.0" "Utilities for working with ISO dates" '((emacs "28.1")) :commit "a957d9ae579eeed146e929dd70e5d1a6e265bd06" :authors '(("Lucas Quintana" . "lmq10@protonmail.com")) :maintainer '("Lucas Quintana" . "lmq10@protonmail.com") :url "https://github.com/lmq-10/iso-date")
